<?php
##### rrshared_modsettingsblocks4.php, installed by several of Robin Richmond's mods
/*
The purpose of this Include file is to define a "Mod Settings Block" for a mod's
mod options. It is one of the "Shared Mod Includes" files stored in the
subfolder RR-shared_mod_includes4, which is distributed with (zipped into)
mods that implement
- Mod Settings Blocks,
- Inner Mod Menus, or
- New Tab flags.

See the TNG Wiki article 
https://tng.lythgoes.net/wiki/index.php?title=Mod_Settings_Blocks
and/or the file ModSettingsBlocks.docx in the mod subfolder

This file contains PHP functions, a stylesheet, and a Javascript block.
The primary functionality of this file is implemented through a Javascript
document.ready function that organizes a mod's options into Mod Setting Block.
It also contains a function that places a small button (with the label "D")
next to a mod option form field. The button simply sets the form field to
a default value (or default state).

************* Revision History ***************
v4 15 Jan 2020
- Can add a new subform to admin_pedconfig.php (Admin>>Setup>>Chart Setting)
  like the new subform in admin_genconfig.php where mods Mod Settings Blocks
  are defined.
- Consolidated the four functions that define default values for mod options
  into one function with a new parameter that specifies the field type
  (selection box, text field, checkbox, textarea)
v3 1 Dec 2019
- (Of course) updated the version number suffixes in the filename and function names.
- Fixed a bug in the code that places a Mod Settings Block in an Admin>>Setup program
  that does not have submenus.
- Added version number suffixes to all Javascript functions, even if they are only called
  internally.
v2 4 Nov 2019
- Added version number suffixes to all PHP functions.
- When a mod settings block is opened, open just that one block. It previously opened
  all block if there were fewer than 4 blocks in the setup program.
- Added a border to the wiki logo hyperlink since the logo didn't look enougth like a button.
- Added the hyperlink cursor to the wiki logo.
- Added PHP functions that define Field Button-like buttons marked with a D to explicitly
  assign default values to options in text and textarea fields. (Version 1 had such buttons
  for checkboxes and selection boxes. These functions are essential redundant with D field
  buttons, but are consistent with the existing default buttons.)
*/

################### External PHP Function ###############
# Generate a button that sets a mod option default in an Admin>>Setup form
function rrModSettingDefault4($fieldtype, $fieldID, $value) {
	$html = "<button class='rrSetupDefaultButton' ";
	if ($fieldtype == 'checkbox')
		$html .= "onclick=\"$('#$fieldID').prop('checked',$value);return false;\"
		title=\"" . ($checked ? 'Check the box' : 'Uncheck the box') . "\">D</button>";
	elseif ($fieldtype == 'select')
		$html .= "<button class='rrSetupDefaultButton'
		onclick=\"$('#$fieldID>option').attr('selected',false); $('#$fieldID>option[value=$value]').attr('selected',true); return false;\"
		title=\"Select '$value'\">D</button>";
	elseif ($fieldtype == 'texarea')
		$html .= "<button class='rrSetupDefaultButton' 
		onclick=\"$('#$fieldID').val($('#$value').html());return false;\"
		title='Set to default value'>D</button>\n";
	elseif ($fieldtype == 'text')
		$html .= "<input type='button' class='rrSetupDefaultButton' 
		onclick=\"$('#$fieldID').val('$value');return false;\"
		title=\" Set to $value\" value='D'>\n";
	else #shouldn't happen
		$html .= ">";
	return $html;
} # End function rrModSettingDefault4


############################################################
############ document.ready function #######################
# This function can be invoked in (almost) any Admin>>Setup program. 
# It reorganizes the "source" Mod Settings Blocks so that, in they end.
# 1. They are inside an HTML layout table that lays the mod options (i.e. form fields)
#    for an Admin>>Setup program's subform, or, for programs without a subform, the main
#    HTML form layout table.
# 2. They are sorted by mod name.
# 3. They are placed inside a new row in the layout table
#    Note that layout table rows usually contain a single form field, but with Mod Settings
#    Blocks, each row contains a table that contains all of the settings for one mod.
# 4. Each layout table row actually contains a bordered div that contains:
#    a. The mod name as a heading,
#    b. A link to the mod's Wiki article,
#    c. The mod's Mod Settings Block.
?>
<script type='text/javascript'>
/////////////////////////////////////////////
$(function() {
	//Nothing to do if there are no settings tables.
	if (!$('.rrConfigTable').length) return false; 

	var configProgram = window.location.pathname.split("/").pop();
	if (configProgram == 'admin_genconfig.php' || configProgram=='admin_pedconfig.php') {
		//Create a new subform for all mod settings that are defined in Mod Settings Block.
		//In admin_genconfig, give it the label "Robin's Mods" and id="robin".
		//In admin_genconfig, give it the label "Other" and id="robin".
		var newFormLabel = "Robin's mods";
		var oldFormId = "misc";
		if (configProgram=='admin_pedconfig.php') {
			newFormLabel = "Other";
			oldFormId = "peddesc";
		}
		var html = "<tr class='databack tngshadow'><td class='tngshadow>";
		html += "<span class='subhead'><a href='#' onclick='return rrToggleRobin4();' class='togglehead' style='color:black'>";
		html += "<img src='img/tng_expand.gif' title='toggle display' alt='toggle display' width='15' height='15' border='0' id='plusrr'>";
		html += "<strong class='th-indent'>"+newFormLabel +"</strong></a></span><br />";
		html += "<div id='robin' style='display:none'><br><table class='normal'><tbody></tbody></table></div>";
		html += "</td></tr>";
		//The new subform follows the last native subform, 'Miscellaneous'
		$("#"+oldFormId).closest('tr').after(html);
	}

	//Sort the options tables that Robin's mods have created.
	var rrGroups = Array();
	var rrMods = Array();
	var rrModOptions = Array();
	var rrModSubform = Array();
	var rrCount = 0; //Counts mod that go in Robin's new subform
	$('.rrConfigTable').each(function() {
		var modname = $(this).attr('modname');
		rrMods.push(modname);
		rrModOptions[modname] = $(this);
		var subform = $(this).attr('subform');
		if (typeof(subform) == "undefined") {
			if (configProgram == 'admin_genconfig.php' || configProgram == 'admin_pedconfig.php') {
				subform = 'robin';
				rrCount++;
			}
			else subform = '';
		}
		rrModSubform[modname] = subform;
		$(this).remove();
	});
	rrMods.sort(); //Sort by modname

	var modToOpen = rrGetQueryStringParam4('mod');
	var modDisplayStyle = '';
	//In admin_genconfig only, hide each mod if a modid has been passed in
	if (modToOpen && (configProgram == 'admin_genconfig.php' || configProgram=='admin_pedconfig.php'))
		modDisplayStyle = " style='display:none;'";

	//Put each mod into the appropriate subform
	for (rr=0; rr<rrMods.length; rr++) {
		var modname = rrMods[rr];
		var modid = modname.replace(/ /g, '');
		var group = rrModOptions[modname].attr('group');
		if (!group) group = 'Misc'; //Not used for now

		html = "<tr"+modDisplayStyle+" tabindex='-1' id='" +modid+ "' class='rrModOptions'>" +
			"<td colspan='5' class='normal'>" +
			"<div style='float:right;'><input type='submit' name='submit' value='Save All'></div>" +
			"<div class='rrModWrapper'>" +
			"<b style='font-size:110%;'>"+modname+"</b> &nbsp;" +
			"<a href='#' onclick=\"return rrModOptionsOpenWikiPage4('" +modname+ "', 'Mod_Options');\">" +
			"<img src='img/rrshared_wikilogo.gif' border='2' style='height:16px;width:16px;' class='nw rrModAdminWiki'></a>" +
			"<table class='rrModTable normal'>" + rrModOptions[modname].html() + "</table>" +
			"</div></td></tr>";

		//Each mod's content is put a new table row at the end of the form's layout table,
		//or, if there is a subform, in the subform's layout table.
		//(The calls to the .first method is probably be unnecessary, but doesn't hurt.)
		var parentElement = $('form');
		if (rrModSubform[modname])
			parentElement = $('#'+rrModSubform[modname]);
		parentElement.children('table').first().children('tbody').first().append(html);
	}

	//Open the subform section specified by the querystring parameter 'open',
	var subform = rrGetQueryStringParam4('open');
	if (subform) {
		if (subform=='robin' || subform=='other') {
			if (modToOpen) {
				$("#"+modToOpen).show(); //Don't toggle the arrow in the heading
				//Open the mod and bring focus to it.
				$("#robin").show();
				$('#'+modToOpen).focus();
			} else { //Open the subform and all mods in it
				rrToggleRobin4();
			}
		} else { //Use the native function to open the subform
			toggleSection(subform, rrGetImageId4(subform), 'on');
		}
	} else if (modToOpen) {
		//bring focus to the mod settings block.
		$('#'+modToOpen).focus();
	} //end if open a subform
}); //end document.ready function

////////////////////////////////////////////////
//Function to toggle the display of the subform for Robin's mods.
//It is different from the generic toggleSection function because each
//mod in the subform must be opened the first time the subform is opened.
function rrToggleRobin4() {
	if ($('#plusrr').attr('src')=='img/tng_expand.gif') {
//	if ($('#robin').css('display')=='block') {
		//Opening the subform. Each mod must be opened, too.
		$('#robin').show();
		$('#plusrr').attr('src', 'img/tng_collapse.gif');
		$('.rrModOptions').show(); //Show each mod
	} else { //Closing the subform 	
		//There is no need to hide each mod
		$('#robin').hide();
		$('#plusrr').attr('src', 'img/tng_expand.gif');
	}
	return false;
} //end function rrToggleRobin4

////////////////////////////////////////////////
//Gets the value of a querystring parameter (duh!)
function rrGetQueryStringParam4(sParam) {
	var sPageURL = window.location.search.substring(1);
	var sURLVariables = sPageURL.split('&');
	for (var i = 0; i < sURLVariables.length; i++) {
		var sParameterName = sURLVariables[i].split('=');
		if (sParameterName[0] == sParam)
			return sParameterName[1];
	}
	return '';
} //end function rrGetQueryStringParam4

///////////////////////////////////////////////
//When the querystring has an "open" parameter, determine the image ID
//associated with that section. This applies only to the config programs
//that have multiple subforms - admin_genconfig.php and admin_pedconfig.php
function rrGetImageId4 (formSection) {
	switch (formSection) {
		case 'ped': return 'plus0';
		case 'desc': return 'plus1';
		case 'rel': return 'plus2';
		case 'time': return 'plus3';
		case 'peddesc': return 'plus4';
		case 'db': return 'plus0';
		case 'tables': return 'plus1';
		case 'folders': return 'plus2';
		case 'site': return 'plus3';
		case 'media': return 'plus4';
		case 'lang': return 'plus5';
		case 'priv': return 'plus6';
		case 'names': return 'plus7';
		case 'cemeteries': return 'plus8';
		case 'mailreg': return 'plus9';
		case 'pref': return 'plus10';
		case 'mobile': return 'plus11';
		case 'dna': return 'plus12';
		case 'misc': return 'plus13';
	} //end switch
} //end function rrGetImageId4

//////////////////////////////////////////////////
//function rrModOptionsOpenWikiPage opens a TNG Wiki page as a popup.
//It is invoked by the Wikipedia icon next the mod name that serves as a heading
//for that mod's options.
function rrModOptionsOpenWikiPage4(article, anchor='') {
	newwindow = window.open('https://tng.lythgoes.net/wiki/index.php?title='+article+'#'+anchor, 'wikiwindow', 'height=600, width=780, top=50, left=100, resizable=yes, scrollbars=yes, status=yes');
	newwindow.focus();
	return false;
} //end function rrModOptionsOpenWikiPage4
</script>

<style>
/************* Styles for Robin's Mod Options **************/
/* (Version number suffixes will need to be added to the classnames if their rules change!?!)
/* The dummy table that contains the options form field definitions before they
are moved into the programs form. */
.rrConfigTable {display:none;}
/* rrModWrap wraps each mod's content */
.rrModWrapper {box-shadow: 0 0 0 2px grey;
	border-radius:5px;
	padding:1px 3px;}
.rrModWrapper h4 {margin:0;}
.rrModWrapper tr {vertical-align:top;}
.rrModWrapper input[type='checkbox'] {position:relative; top:2px;}
.rrModWrapper input[type='radio'] {position:relative; top:2px;}
.rrModTable {clear:both;}
/* Right justify the option field prompts, but not mod names, which are the
first elements in each row of a subordinate table. */
.rrModTable>tbody>tr>td:first-child {text-align:right;
	padding-right:4px; padding-top:4px;}
/* Little field-button-like buttons to set option default values */
.rrSetupDefaultButton {padding:1px;
	font-size:95%;
	font-weight:bold;
	border-radius:3px;}
/* For the wiki logo in the block headings */
.rrModAdminWiki {cursor:pointer;}
</style>

